/*
 * File:   system_init.c
 * Author: yts
 *
 * Created on 2017/10/04, 9:05
 */


// DSPIC33FJ128GP802 Configuration Bit Settings

// 'C' source line config statements

// FBS
#pragma config BWRP = WRPROTECT_OFF     // Boot Segment Write Protect (Boot Segment may be written)
#pragma config BSS = NO_FLASH           // Boot Segment Program Flash Code Protection (No Boot program Flash segment)
#pragma config RBS = NO_RAM             // Boot Segment RAM Protection (No Boot RAM)

// FSS
#pragma config SWRP = WRPROTECT_OFF     // Secure Segment Program Write Protect (Secure segment may be written)
#pragma config SSS = NO_FLASH           // Secure Segment Program Flash Code Protection (No Secure Segment)
#pragma config RSS = NO_RAM             // Secure Segment Data RAM Protection (No Secure RAM)

// FGS
#pragma config GWRP = OFF               // General Code Segment Write Protect (User program memory is not write-protected)
#pragma config GSS = OFF                // General Segment Code Protection (User program memory is not code-protected)

// FOSCSEL
#pragma config FNOSC = PRIPLL           // Oscillator Mode (Primary Oscillator (XT, HS, EC) w/ PLL)
#pragma config IESO = ON                // Internal External Switch Over Mode (Start-up device with FRC, then automatically switch to user-selected oscillator source when ready)

// FOSC
#pragma config POSCMD = XT              // Primary Oscillator Source (XT Oscillator Mode)
#pragma config OSCIOFNC = OFF           // OSC2 Pin Function (OSC2 pin has clock out function)
#pragma config IOL1WAY = ON             // Peripheral Pin Select Configuration (Allow Only One Re-configuration)
#pragma config FCKSM = CSECME           // Clock Switching and Monitor (Both Clock Switching and Fail-Safe Clock Monitor are enabled)

// FWDT
#pragma config WDTPOST = PS32768        // Watchdog Timer Postscaler (1:32,768)
#pragma config WDTPRE = PR128           // WDT Prescaler (1:128)
#pragma config WINDIS = OFF             // Watchdog Timer Window (Watchdog Timer in Non-Window mode)
#pragma config FWDTEN = OFF             // Watchdog Timer Enable (Watchdog timer enabled/disabled by user software)

// FPOR
#pragma config FPWRT = PWR128           // POR Timer Value (128ms)
#pragma config ALTI2C = OFF             // Alternate I2C  pins (I2C mapped to SDA1/SCL1 pins)

// FICD
#pragma config ICS = PGD1               // Comm Channel Select (Communicate on PGC1/EMUC1 and PGD1/EMUD1)
#pragma config JTAGEN = OFF             // JTAG Port Enable (JTAG is Disabled)

// #pragma config statements should precede project file includes.
// Use project enums instead of #define for ON and OFF.

#include <xc.h>
#include <dsp.h>

unsigned int BufferA[32] __attribute__((space(dma)));
unsigned int BufferB[32] __attribute__((space(dma)));

// DMA initialization
void initDMA1(void){
    DMA1CON = 0;            // Clear the registor
    DMA1CONbits.MODE = 2;   // Configure DMA for Continuous Ping-Pong mode

    DMA1REQ = 0x000d;// Select ADC1 as DMA Request source
        
    DMA1STA = __builtin_dmaoffset(BufferA);
    DMA1STB = __builtin_dmaoffset(BufferB);
    
    DMA1PAD = (volatile unsigned int)&ADC1BUF0; // Point DMA to ADC1BUF0
    DMA1CNT = 31;// 32 DMA request

    IFS0bits.DMA1IF = 0;    //Clear the DMA interrupt flag bit
    IEC0bits.DMA1IE = 1;    //Set the DMA interrupt enable bit

    DMA1CONbits.CHEN=1;     // Enable DMA channel
   
}

// ADC initialization
void initAdc1(void)
{

    AD1CON1bits.ADSIDL = 0; //continue operation in idle mode
    AD1CON1bits.ADDMABM = 1;//no meaning; no dma
    AD1CON1bits.AD12B = 1;  //12bit
    AD1CON1bits.FORM = 0;   //unsigned int 0x000-0xFFF
    AD1CON1bits.SSRC = 2;   //timer3 is used for a sample clock source
    AD1CON1bits.ASAM = 1;   //auto start sampling
    
    AD1CON2bits.VCFG = 0;   //Add abd AVss pins are used
    AD1CON2bits.CSCNA = 0;  //no scan
    AD1CON2bits.SMPI = 0;   //CH0 is sellected
    AD1CON2bits.BUFM = 0;   //buffer fill from the start address
    AD1CON2bits.ALTS = 0;   //Sample A

/*************************************************
 * Tcy = 1 / (40 MHz) = 25 ns  
 * Tad = Tcy * (ADCS + 1) >117.6 
 * ADCS > 3.7 
 * ADCS = 4
 * sampling time 4Tad;  conversion time 14Tad
 * total time  17 Tad = 17 * 125 ns = 2.125 usec
 *************************************************/
    
    AD1CON3bits.ADRC = 0;   //system clock is used
    AD1CON3bits.SAMC = 3;   //auto sample time 3 Tad
    AD1CON3bits.ADCS = 4;   //conversion clock  Tad = 25 * (4 + 1)
    
    AD1CHS0bits.CH0NA = 0;  //Vrefl :negative input
    AD1CHS0bits.CH0SA = 9;  //AN9 : positive input

    AD1CSSL = 0;            //no scan 
    AD1PCFGLbits.PCFG9 = 0; //AN9 : analog input

    AD1CON1bits.ADON = 1;   //ADC start
    
//	ConfigIntADC1(ADC_INT_PRI_5 & ADC_INT_ENABLE);  
    
    
//  IFS0bits.AD1IF = 0;		//Clear ADC interrupt flag
//	IEC0bits.AD1IE = 1;	// Enable ADC Interrupts 
//  IPC3bits.AD1IP = 4;	// ADC interrupt priority
}

void initTmr3() 
{
    T3CONbits.TSIDL = 0;    //Continue in idle mode
    T3CONbits.TGATE = 0;    //Disable Gated Timer mode    
    T3CONbits.TCKPS = 3;    //Select 1:256  Prescaler
    T3CONbits.TCS = 0;		// Select internal instruction cycle clock 
    
    TMR3 = 0x00; // Clear 16-bit Timer
    PR3 = 3906 - 1; // Load 16-bit period value (~25 msec in case 40 MHz)
    
//    IPC2bits.T3IP = 4;      // Set Timer2 Interrupt Priority Level
//    IFS0bits.T3IF = 0;      // Clear Timer2 Interrupt Flag
//    IEC0bits.T3IE = 1;      // Enable Timer2 interrupt
          
    T3CONbits.TON =1;       //Start 16-bit Timer

    //Equivalent setting:    
	//OpenTimer3(T3_ON & T3_IDLE_CON & T3_GATE_OFF & T3_PS_1_256 & T3_SOURCE_INT, 39063 - 1);  //40MHz 250msec
	//ConfigIntTimer3(T3_INT_PRIOR_4 & T3_INT_ON);
}


unsigned int DmaBuffer = 0;
unsigned int data[32];


void ProcessADCSamples(unsigned int *buffer){

    }



int aa=0;

void __attribute__((interrupt, no_auto_psv)) _DMA1Interrupt(void)
{
    // Switch between Primary and Secondary Ping-Pong buffers
    if(DmaBuffer == 0)
        ProcessADCSamples(BufferA);
    else
        ProcessADCSamples(BufferB);
    
    DmaBuffer ^= 1;

    aa++;
    if(aa==2){
        aa=0;
    }
    
    IFS0bits.DMA1IF = 0;//Clear the DMA1 Interrupt Flag
}


void SYS_Initialize ( void )
{
    // Clock settings with 10 MHz external X-tal: 40MHz operation
    // Fosc= Fin * M / (N1 * N2), Fcy = Fosc / 2
    // Fosc= 10 MHz * 32 / (2 * 2) = 80 MHz, Fcy = Fosc/2 = 40MHz
    CLKDIVbits.PLLPRE = 0;      	// N2=2 (1,2,4,8,16,32,64,256) 0.8-8 MHz
    PLLFBD = 30;               		// M=32 (2-33) 100-200 MHz
    CLKDIVbits.PLLPOST = 0;     	// N1=2 (2,4,8) 12.5-80 MHz

    while(OSCCONbits.LOCK !=1 );    //wait for PLL-lock
        
   	initAdc1();             	// Initialize ADC
	initTmr3();					// Initialise TIMER 3    
    initDMA1();
}